/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/

#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>

/****************************************************************************
 *  Tests 3D ellipses                                                       *
 ***************************************************************************/
void draw_ellipse(Display          *dpy,
               XID              resourceID,
               PEXOCRequestType req_type)

  {
    PEXCoord    center, pts2[2];
    PEXVector   major, minor;
    PEXColor    edge_color, line_color, int_color;

    /*****************************************************************
     Top Row. Draws basic ellipses with default attributes
    *****************************************************************/

    center.x = 0.20;
    center.y = 0.8;
    center.z = 0.0;

    major.x = 0.15;
    major.y = 0.0;
    major.z = 0.0;

    minor.x = 0.0;
    minor.y = 0.05;
    minor.z = 0.0;

    line_color.rgb.red = 1.0;
    line_color.rgb.green = 0.0;
    line_color.rgb.blue = 0.0;
    PEXSetLineColor(dpy, resourceID, req_type, PEXColorTypeRGB, &line_color);
    PEXSetLineType (dpy, resourceID, req_type, PEXLineTypeDashed);
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    center.x = 0.55;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    center.x = 0.79;
    major.x = 0.05;
    minor.y = 0.15;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    center.x = 0.92;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);
  
    /*****************************************************************
     Middle Row.                                              
    *****************************************************************/
    int_color.rgb.red = 1.0;
    int_color.rgb.green = 0.0;
    int_color.rgb.blue = 0.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &int_color);
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleSolid);
    center.x = 0.5;
    center.y = 0.5;
    center.z = 0.0;
    major.x = 0.15;
    major.y = 0.15;
    major.z = 0.0;
    minor.x = 0.05;
    minor.y = -0.05;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    int_color.rgb.red = 0.0;
    int_color.rgb.green = 1.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &int_color);
    major.x = -0.15;
    major.y = 0.15;
    minor.x = 0.05;
    minor.y = 0.05;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    int_color.rgb.green = 0.0;
    int_color.rgb.blue = 1.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB, 
      &int_color);
    major.x = 0.0;
    major.y = 0.2;
    minor.x = 0.0;
    minor.y = 0.0;
    minor.z = 0.05;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    int_color.rgb.green = 1.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB,
      &int_color);
    major.x = 0.2;
    major.y = 0.0;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);
    
    /*****************************************************************
     Bottom Row. Ellipses drawn with different area attributes
    *****************************************************************/
    
    int_color.rgb.blue = 0.0;
    PEXSetSurfaceColor(dpy, resourceID, req_type, PEXColorTypeRGB,
      &int_color);
    PEXSetSurfaceEdgeFlag(dpy, resourceID, req_type, PEXOn);
    center.x = 0.20;
    center.y = 0.2;
    major.x = 0.15;
    major.y = 0.0;
    major.z = 0.0;
    minor.x = 0.0;
    minor.y = 0.05;
    minor.z = 0.0;
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    PEXSetSurfaceEdgeType(dpy, resourceID, req_type, PEXSurfaceEdgeDashed);
    center.x = 0.55;
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleHatch); 
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);

    center.x = 0.85;
    major.x = 0.1;
    major.y = 0.1;
    major.z = 0.0;
    minor.x = 0.05;
    minor.y = -0.05;
    edge_color.rgb.red = 1.0;
    edge_color.rgb.green = 0.0;
    edge_color.rgb.blue = 0.0;
    PEXSetSurfaceEdgeColor(dpy, resourceID, req_type,PEXColorTypeRGB, 
      &edge_color);
    PEXSetInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleSolid); 
    PEXSetBFInteriorStyle(dpy, resourceID, req_type, PEXInteriorStyleHatch);
    PEXExtEllipse(dpy, resourceID, req_type, &center, &major, &minor);
  }

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


  {
    strcpy(test_name, "ellipse");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "Ellipse3D");
  }


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
  {
  }

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
  {

    describe_test( "\n" );
    describe_test( " Tests 3D ellipses with different Attributes:\n");
    describe_test( "\n" );
    describe_test( " TOP ROW: 4 3D ellipses are drawn with default attributes.\n");
    describe_test( "          They should be white with hollow interior.\n");
    describe_test( " MIDDLE ROW: Four ellipses are drawn. The interior style is set to solid.\n");
    describe_test( "             There should be two inclined ellipses with Red and Green interior.\n");
    describe_test( "             One vertical blue line which is an ellipse with a minor axis along\n");
    describe_test( "             the z-axis.\n");
    describe_test( "             One Horizontal Cyan line which is also an ellipse.\n");
    describe_test( " BOTTOM ROW: Three eliipses. The first one should have a green interior\n");
    describe_test( "             with a white edge.\n");
    describe_test( "             The second one has an hatch interior with dashed surface edge.\n");
    describe_test( "             The last one is drawm with a Red dashed edge and green interior.\n");
    describe_test( "\n");
    draw_ellipse(dpy, resourceID, req_type);

  }

void testproc(Display *dpy, Window window, int cur_image)
  {
    image_testproc("ellipse", "Ellipse 3D", img_desc,
                    dpy,window, 0, 0,
                    glob_window_width, glob_window_height);
  }

