/******************************************************************************/
/*  (C) COPYRIGHT International Business Machines Corp. 1993                  */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THIS MATERIAL        */
/*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,        */
/*  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY  */
/*  AND FITNESS FOR A PARTICULAR PURPOSE.                                     */
/*                                                                            */
/******************************************************************************/
#include <X11/PEX5/PEXlib.h>
#include <X11/PEX5/PEXExtlib.h>
#include <misc.h>

Window local_window;
/****************************************************************************
 *   No primitives drawn in this test                                       *
 ***************************************************************************/
void draw_model(Display          *dpy,
               XID              resourceID,
               PEXOCRequestType req_type)
{
    unsigned long            mask[2];
    unsigned long            pc_mask[3] = {0, 0, 0};
    PEXExtRendererAttributes attrs, *rcv_attrs; 
    PEXPCAttributes          pc_attr;
    PEXPipelineContext       pc;
    unsigned long            num_paths;
    PEXDeviceRect            rects;

    PEXExtSetRendererAttributeMaskAll(mask);

    mask[0] &= ~PEXRACurrentPath; 
    mask[0] &= ~PEXRAPickStartPath;
    mask[0] &= ~PEXRARendererState;
    mask[1] = 0;
/* Define the bundle tables */

    attrs.marker_bundle =
	PEXCreateLookupTable( dpy, local_window, PEXLUTMarkerBundle );
    attrs.text_bundle =
	PEXCreateLookupTable( dpy, local_window, PEXLUTTextBundle );
    attrs.line_bundle =
	PEXCreateLookupTable( dpy, local_window, PEXLUTLineBundle );
    attrs.interior_bundle =
	PEXCreateLookupTable( dpy, local_window, PEXLUTInteriorBundle );
    attrs.edge_bundle =
	PEXCreateLookupTable( dpy, local_window, PEXLUTEdgeBundle );
    attrs.view_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTView );
    attrs.color_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTColor );
    attrs.depth_cue_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTDepthCue );
    attrs.light_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTLight );
    attrs.color_approx_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTColorApprox );
    attrs.pattern_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTPattern );
    attrs.text_font_table =
	PEXCreateLookupTable( dpy, local_window, PEXLUTTextFont );
  
/* Define the incl/excl filters */
  
    attrs.highlight_incl = PEXCreateNameSet( dpy );
    attrs.highlight_excl = PEXCreateNameSet( dpy );
    attrs.invisibility_incl = PEXCreateNameSet( dpy );
    attrs.invisibility_excl = PEXCreateNameSet( dpy );
    attrs.pick_incl = PEXCreateNameSet( dpy );
    attrs.pick_excl = PEXCreateNameSet( dpy );
  
/* Misc settings */
  
    attrs.clear_image = True;
    attrs.clear_z = True;
    attrs.echo_mode = PEXEcho;

    attrs.background_color.type = PEXColorTypeRGB;
    attrs.background_color.value.rgb.red = 0.5;  
    attrs.background_color.value.rgb.green = 0.6;  
    attrs.background_color.value.rgb.blue = 0.7;  

/* Define a pipeline context */
  
    PEXSetPCAttributeMask(pc_mask, PEXPCLineType);
    pc_attr.line_type = PEXLineTypeDashed;
    pc = PEXCreatePipelineContext(dpy, pc_mask, &pc_attr);
    attrs.pipeline_context = pc; 
  
/* Define the viewport */
  
    attrs.viewport.min.x = 50; attrs.viewport.max.x = 250;
    attrs.viewport.min.y = 50; attrs.viewport.max.y = 250;
    attrs.viewport.min.z = 50; attrs.viewport.max.z = 250;
    attrs.viewport.use_drawable = False;
  
/* Define the npc subvolume */
  
    attrs.npc_subvolume.min.x = 0.2; attrs.npc_subvolume.max.x = 0.94;
    attrs.npc_subvolume.min.y = 0.2; attrs.npc_subvolume.max.y = 0.94;
    attrs.npc_subvolume.min.z = 0.2; attrs.npc_subvolume.max.z = 0.94;
  
/* Define a clip list */

    rects.xmin = 45.0; rects.xmax = 245.0; 
    rects.ymin = 45.0; rects.ymax = 245.0; 
    attrs.clip_list.count = 1;
    attrs.clip_list.rectangles = &rects;

    attrs.tm_binding_table = 
        PEXCreateLookupTable(dpy, local_window, PEXExtRATMBindingTable); 
    attrs.tm_coord_source_table = 
        PEXCreateLookupTable(dpy, local_window, PEXExtRATMCoordSourceTable); 
    attrs.tm_composition_table = 
        PEXCreateLookupTable(dpy, local_window, PEXExtRATMCompositionTable); 
    attrs.tm_sampling_table = 
        PEXCreateLookupTable(dpy, local_window, PEXExtRATMSamplingTable); 
  
    PEXExtChangeRenderer(dpy, resourceID, mask, &attrs);

/*  Ok, now lets see if we get everything back from the renderer */

    rcv_attrs = PEXExtGetRendererAttributes(dpy, resourceID, mask);

    if (rcv_attrs == NULL) {
       printf("Server error! PEXExtGetRendererAttributes returned a NULL pointer!\n");
       exit(0);
     }

    if (rcv_attrs->marker_bundle != attrs.marker_bundle) 
       _ibmpex_stderr_print("ERROR: Marker Bundle table not set correctly!\n");
    else _ibmpex_stderr_print("Marker Bundle set correctly!\n");

    if (rcv_attrs->text_bundle != attrs.text_bundle) 
       _ibmpex_stderr_print("ERROR: Text Bundle table not set correctly!\n");
    else _ibmpex_stderr_print("Text Bundle set correctly!\n");

    if (rcv_attrs->line_bundle != attrs.line_bundle) 
       _ibmpex_stderr_print("ERROR: Line Bundle table not set correctly!\n");
    else _ibmpex_stderr_print("Line Bundle set correctly!\n");  

    if (rcv_attrs->interior_bundle != attrs.interior_bundle) 
       _ibmpex_stderr_print("ERROR: Interior Bundle table not set correctly!\n");
    else _ibmpex_stderr_print("Interior Bundle set correctly!\n"); 

    if (rcv_attrs->edge_bundle != attrs.edge_bundle) 
       _ibmpex_stderr_print("ERROR: Edge Bundle table not set correctly!\n");
    else _ibmpex_stderr_print("Edge Bundle set correctly!\n");

    if (rcv_attrs->view_table != attrs.view_table) 
       _ibmpex_stderr_print("ERROR: View Table table not set correctly!\n");
    else _ibmpex_stderr_print("View Table set correctly!\n");

    if (rcv_attrs->view_table != attrs.view_table) 
       _ibmpex_stderr_print("ERROR: View Table not set correctly!\n");
    else _ibmpex_stderr_print("View Table set correctly!\n");

    if (rcv_attrs->color_table != attrs.color_table) 
       _ibmpex_stderr_print("ERROR: Color Table not set correctly!\n");
    else _ibmpex_stderr_print("Color Table set correctly!\n");

    if (rcv_attrs->depth_cue_table != attrs.depth_cue_table) 
       _ibmpex_stderr_print("ERROR: Depth Cue Table not set correctly!\n");
    else _ibmpex_stderr_print("Depth Cue Table set correctly!\n");

    if (rcv_attrs->light_table != attrs.light_table) 
       _ibmpex_stderr_print("ERROR: Light Table not set correctly!\n");
    else _ibmpex_stderr_print("Light Table set correctly!\n");

    if (rcv_attrs->color_approx_table != attrs.color_approx_table) 
       _ibmpex_stderr_print("ERROR: Color Approx Table not set correctly!\n");
    else _ibmpex_stderr_print("Color Approx Table set correctly!\n");

    if (rcv_attrs->pattern_table != attrs.pattern_table) 
       _ibmpex_stderr_print("ERROR: Pattern Table not set correctly!\n");
    else _ibmpex_stderr_print("Pattern Table set correctly!\n");

    if (rcv_attrs->text_font_table != attrs.text_font_table) 
       _ibmpex_stderr_print("ERROR: Text Font Table table not set correctly!\n");
    else _ibmpex_stderr_print("Text Font Table set correctly!\n");

    if (rcv_attrs->tm_binding_table != attrs.tm_binding_table) 
       _ibmpex_stderr_print("ERROR: TM Binding table not set correctly!\n");
    else _ibmpex_stderr_print("TM Binding table set correctly!\n");

    if (rcv_attrs->tm_coord_source_table != attrs.tm_coord_source_table) 
       _ibmpex_stderr_print("ERROR: TM Coord Source table not set correctly!\n");
    else _ibmpex_stderr_print("TM Coord Source table set correctly!\n");

    if (rcv_attrs->tm_composition_table != attrs.tm_composition_table) 
       _ibmpex_stderr_print("ERROR: TM Composition table not set correctly!\n");
    else _ibmpex_stderr_print("TM Composition table set correctly!\n");

    if (rcv_attrs->tm_sampling_table != attrs.tm_sampling_table) 
       _ibmpex_stderr_print("ERROR: TM Sampling table not set correctly!\n");
    else _ibmpex_stderr_print("TM Sampling table set correctly!\n");

    if (rcv_attrs->highlight_incl != attrs.highlight_incl) 
       _ibmpex_stderr_print("ERROR: Highlight Incl filter not set correctly!\n");
    else _ibmpex_stderr_print("Highlight Incl set correctly!\n");

    if (rcv_attrs->highlight_excl != attrs.highlight_excl) 
       _ibmpex_stderr_print("ERROR: Highlight Excl filter table not set correctly!\n");
    else _ibmpex_stderr_print("Highlight Excl filter set correctly!\n");

    if (rcv_attrs->invisibility_incl != attrs.invisibility_incl) 
       _ibmpex_stderr_print("ERROR: Invisibility Incl filter not set correctly!\n");
    else _ibmpex_stderr_print("Invisibility Incl filter set correctly!\n");

    if (rcv_attrs->invisibility_excl != attrs.invisibility_excl) 
       _ibmpex_stderr_print("ERROR: Invisibility Excl filter not set correctly!\n");
    else _ibmpex_stderr_print("Invisibility Excl filter set correctly!\n");

    if (rcv_attrs->pick_incl != attrs.pick_incl) 
       _ibmpex_stderr_print("ERROR: Pick Incl filter not set correctly!\n");
    else _ibmpex_stderr_print("Pick Incl filter set correctly!\n");

    if (rcv_attrs->pick_excl != attrs.pick_excl) 
       _ibmpex_stderr_print("ERROR: Pick Excl filter not set correctly!\n");
    else _ibmpex_stderr_print("Pick Excl filter set correctly!\n");

    if (rcv_attrs->clear_image != attrs.clear_image) 
       _ibmpex_stderr_print("ERROR: clear_image not set correctly!\n");
    else _ibmpex_stderr_print("clear_image set correctly!\n");

    if (rcv_attrs->clear_z != attrs.clear_z) 
       _ibmpex_stderr_print("ERROR: Clear Z not set correctly!\n");
    else _ibmpex_stderr_print("Clear Z set correctly!\n");

    if (rcv_attrs->echo_mode != attrs.echo_mode) 
       _ibmpex_stderr_print("ERROR: Echo Mode not set correctly!\n");
    else _ibmpex_stderr_print("Echo Mode set correctly!\n");

    if (rcv_attrs->background_color.type != attrs.background_color.type) 
       _ibmpex_stderr_print("ERROR: background_color type not set correctly!\n");
    else _ibmpex_stderr_print("background_color type set correctly!\n");

    if (rcv_attrs->background_color.value.rgb.red != attrs.background_color.value.rgb.red) 
       _ibmpex_stderr_print("ERROR: background_color red not set correctly!\n");
    else _ibmpex_stderr_print("background_color red set correctly!\n");

    if (rcv_attrs->background_color.value.rgb.green != attrs.background_color.value.rgb.green) 
       _ibmpex_stderr_print("ERROR: background_color green not set correctly!\n");
    else _ibmpex_stderr_print("background_color green set correctly!\n");

    if (rcv_attrs->background_color.value.rgb.blue != attrs.background_color.value.rgb.blue) 
       _ibmpex_stderr_print("ERROR:background_color blue not set correctly!\n");
    else _ibmpex_stderr_print("background_color blue set correctly!\n");

    if (rcv_attrs->pipeline_context != attrs.pipeline_context) 
       _ibmpex_stderr_print("ERROR: Pipeline Context not set correctly!\n");
    else _ibmpex_stderr_print("Pipeline Context set correctly!\n");

    if (rcv_attrs->viewport.min.x != attrs.viewport.min.x) 
       _ibmpex_stderr_print("ERROR: Viewport min x not set correctly!\n");
    else _ibmpex_stderr_print("Viewport min x set correctly!\n");

    if (rcv_attrs->viewport.max.x != attrs.viewport.max.x) 
       _ibmpex_stderr_print("ERROR: Viewport max x not set correctly!\n");
    else _ibmpex_stderr_print("Viewport max x set correctly!\n");

    if (rcv_attrs->viewport.min.y != attrs.viewport.min.y) 
       _ibmpex_stderr_print("ERROR: Viewport min y not set correctly!\n");
    else _ibmpex_stderr_print("Viewport min y set correctly!\n");

    if (rcv_attrs->viewport.max.y != attrs.viewport.max.y) 
       _ibmpex_stderr_print("ERROR: Viewport max y not set correctly!\n");
    else _ibmpex_stderr_print("Viewport max y set correctly!\n");

    if (rcv_attrs->viewport.min.z != attrs.viewport.min.z) 
       _ibmpex_stderr_print("ERROR: Viewport min z not set correctly!\n");
    else _ibmpex_stderr_print("Viewport min z set correctly!\n");

    if (rcv_attrs->viewport.max.z!= attrs.viewport.max.z) 
       _ibmpex_stderr_print("ERROR: Viewport max z not set correctly!\n");
    else _ibmpex_stderr_print("Viewport max z set correctly!\n");

    if (rcv_attrs->viewport.use_drawable != attrs.viewport.use_drawable) 
       _ibmpex_stderr_print("ERROR: Viewport use drawable not set correctly!\n");
    else _ibmpex_stderr_print("Viewport use drawable set correctly!\n");
 
    if (rcv_attrs->npc_subvolume.min.x != attrs.npc_subvolume.min.x) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume min x not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume min x set correctly!\n");

    if (rcv_attrs->npc_subvolume.max.x != attrs.npc_subvolume.max.x) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume max x not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume max x set correctly!\n");

    if (rcv_attrs->npc_subvolume.min.y != attrs.npc_subvolume.min.y) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume min y not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume min y set correctly!\n");

    if (rcv_attrs->npc_subvolume.max.y != attrs.npc_subvolume.max.y) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume max y not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume max y set correctly!\n");

    if (rcv_attrs->npc_subvolume.min.z != attrs.npc_subvolume.min.z) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume min z not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume min z set correctly!\n");

    if (rcv_attrs->npc_subvolume.max.z!= attrs.npc_subvolume.max.z) 
       _ibmpex_stderr_print("ERROR: NPC Subvolume max z not set correctly!\n");
    else _ibmpex_stderr_print("NPC Subvolume max z set correctly!\n");


    if (rcv_attrs->clip_list.rectangles->xmin != attrs.clip_list.rectangles->xmin) 
       _ibmpex_stderr_print("ERROR: Clip List Rectangles xmin not set correctly!\n");
    else _ibmpex_stderr_print("Clip List Rectangles xmin set correctly!\n");

    if (rcv_attrs->clip_list.rectangles->xmax != attrs.clip_list.rectangles->xmax) 
       _ibmpex_stderr_print("ERROR: Clip List Rectangles xmax not set correctly!\n");
    else _ibmpex_stderr_print("Clip List Rectangles xmax set correctly!\n");

    if (rcv_attrs->clip_list.rectangles->ymin != attrs.clip_list.rectangles->ymin) 
       _ibmpex_stderr_print("ERROR: Clip List Rectangles ymin not set correctly!\n");
    else _ibmpex_stderr_print("Clip List Rectangles ymin set correctly!\n");

    if (rcv_attrs->clip_list.rectangles->ymax != attrs.clip_list.rectangles->ymax) 
       _ibmpex_stderr_print("ERROR: Clip List Rectangles ymax not set correctly!\n");
    else _ibmpex_stderr_print("Clip List Rectangles ymax set correctly!\n");

    if (rcv_attrs->clip_list.count != attrs.clip_list.count) 
       _ibmpex_stderr_print("ERROR: Clip List Count not set correctly!\n");
    else _ibmpex_stderr_print("Clip List Count set correctly!\n");

    PEXExtFreeRendererAttributes(rcv_attrs);
}

/****************************************************************************
 *  Sets the test parameters                                                *
 ***************************************************************************/
void inquire_test_params(char *test_name,
                         int  *num_images,
                         int  *supported_modes,
                         char *win_title)


{
    strcpy(test_name, "Get Renderer Attributes Test");
    *num_images = 1;
    *supported_modes = IMM_MODE | STR_MODE;

    strcpy(win_title, "PEX Get Renderer Attributes Test");
}


/****************************************************************************
 *  This routine is used to do miscellaneous things like setting some       *
 *  renderer attributess, Viewing etc if needed.                            *
 ***************************************************************************/
void misc_setup(Display *dpy, Window window, PEXRenderer renderer,
                int cur_image)
{
   local_window = window;
   _ibmpex_set_stderr("getrend.txt", "getrend");
}

/****************************************************************************
 *  This routine calls the function which does the actual test              *
 ***************************************************************************/
void execute_test(Display *dpy, XID resourceID,
               PEXOCRequestType req_type, int cur_image)
{
    describe_test( "\n" );
    describe_test( " IBM Renderer Request Test:\n");
    describe_test( "\n" );
    describe_test( " This test will perform the following:\n");
    describe_test( "\n"); 
    describe_test( "     1) Set all extended renderer attribute mask values.\n");
    describe_test( "     2) Call the PEXExtChangeRenderer routine to set the new\n"); 
    describe_test( "     3) Call the PEXExtGetRendererAttributes routine to check the new\n"); 
    describe_test( "        renderer attributes.\n\n"); 

    draw_model(dpy, resourceID, req_type);

}

void testproc(Display *dpy, Window window, int cur_image)
{
    file_testproc("getrend.txt", "getrend");
}

